@@ -119,6 +119,7 @@ enum OutputFormat {
     Human,
     Json,
     Yaml,
+    Markdown,
 }
 
 pub fn run(ctx: &Context, args: ShoppingListArgs) -> Result<()> {
@@ -306,6 +307,9 @@ pub fn run(ctx: &Context, args: ShoppingListArgs) -> Result<()> {
         )?;
     }
 
+    // Use common names from aisle configuration
+    list = list.use_common_names(&aisle, PARSER.converter());
+
     // Subtract pantry quantities from shopping list
     if let Some(pantry_conf) = &pantry {
         list = list.subtract_pantry(pantry_conf, PARSER.converter());
@@ -336,6 +340,10 @@ pub fn run(ctx: &Context, args: ShoppingListArgs) -> Result<()> {
                         list.into_iter().map(|(ingredient, _)| ingredient).collect();
                     serde_yaml::to_writer(w, &ingredients)?;
                 }
+                OutputFormat::Markdown => {
+                    let value = build_md_value(list, &aisle, args.plain, args.ingredients_only);
+                    write!(w, "{value}")?;
+                }
             }
         } else {
             match format {
@@ -356,6 +364,10 @@ pub fn run(ctx: &Context, args: ShoppingListArgs) -> Result<()> {
 
                     serde_yaml::to_writer(w, &value)?;
                 }
+                OutputFormat::Markdown => {
+                    let value = build_md_value(list, &aisle, args.plain, args.ingredients_only);
+                    write!(w, "{value}")?;
+                }
             }
         }
         Ok(())
@@ -401,6 +413,46 @@ fn build_human_table(list: IngredientList, aisle: &AisleConf, plain: bool) -> ta
     table
 }
 
+fn build_md_value(
+    list: IngredientList,
+    aisle: &AisleConf,
+    plain: bool,
+    ingredients_only: bool,
+) -> String {
+    let mut output = String::new();
+
+    let format_ingredient = |ingredient: &str, quantity: &GroupedQuantity| {
+        if ingredients_only {
+            format!("- {ingredient}\n")
+        } else {
+            let quantity_string = quantity
+                .iter()
+                .map(quantity_fmt)
+                .collect::<Vec<_>>()
+                .join(", ");
+            format!("- *{quantity_string}* {ingredient}\n")
+        }
+    };
+    if plain {
+        // no categories, simple list
+        for (ingredient, quantity) in list {
+            output.push_str(&format_ingredient(&ingredient, &quantity));
+        }
+    } else {
+        let categories = list.categorize(aisle);
+        for (i, (category, items)) in categories.into_iter().enumerate() {
+            if i > 0 {
+                output.push('\n');
+            }
+            output.push_str(&format!("# {category}\n"));
+            for (ingredient, quantity) in items {
+                output.push_str(&format_ingredient(&ingredient, &quantity));
+            }
+        }
+    }
+    output
+}
+
 fn build_json_value<'a>(
     list: IngredientList,
     aisle: &'a AisleConf<'a>,
