@@ -266,7 +266,11 @@ fn ingredients(w: &mut impl io::Write, recipe: &Recipe, converter: &Converter) -
         return Ok(());
     }
     writeln!(w, "Ingredients:")?;
-    let mut table = Table::new("  {:<} {:<}    {:<} {:<} {:<}");
+
+    // Group ingredients by display name and merge quantities
+    let mut grouped: HashMap<String, (cooklang::quantity::GroupedQuantity, Vec<&Ingredient>)> =
+        HashMap::new();
+
     for entry in recipe.group_ingredients(converter) {
         let GroupedIngredient {
             ingredient: igr,
@@ -274,12 +278,34 @@ fn ingredients(w: &mut impl io::Write, recipe: &Recipe, converter: &Converter) -
             ..
         } = entry;
 
+        let display_name = igr.display_name().to_string();
+        grouped
+            .entry(display_name)
+            .and_modify(|(merged_qty, igrs)| {
+                merged_qty.merge(&quantity, converter);
+                igrs.push(igr);
+            })
+            .or_insert_with(|| (quantity.clone(), vec![igr]));
+    }
+
+    // Sort by name for consistent display
+    let mut sorted_ingredients: Vec<_> = grouped.into_iter().collect();
+    sorted_ingredients.sort_by(|a, b| a.0.cmp(&b.0));
+
+    let mut table = Table::new("  {:<} {:<}    {:<} {:<} {:<}");
+    for (display_name, (quantity, ingredients)) in sorted_ingredients {
         let outcome_style = yansi::Style::new();
         let outcome_char = "";
 
-        let mut row = Row::new().with_cell(igr.display_name());
+        let mut row = Row::new().with_cell(display_name);
 
-        if igr.reference.is_some() {
+        // Show reference if any ingredient has one
+        let has_reference = ingredients.iter().any(|igr| igr.reference.is_some());
+        if has_reference {
+            let igr = ingredients
+                .iter()
+                .find(|igr| igr.reference.is_some())
+                .unwrap();
             let sep = std::path::MAIN_SEPARATOR.to_string();
             let path = igr.reference.as_ref().unwrap().components.join(&sep);
             row.add_ansi_cell(
@@ -289,7 +315,9 @@ fn ingredients(w: &mut impl io::Write, recipe: &Recipe, converter: &Converter) -
             row.add_cell("");
         }
 
-        if igr.modifiers().is_optional() {
+        // Mark as optional only if ALL occurrences are optional
+        let all_optional = ingredients.iter().all(|igr| igr.modifiers().is_optional());
+        if all_optional {
             row.add_ansi_cell("(optional)".paint(styles().opt_marker));
         } else {
             row.add_cell("");
@@ -303,8 +331,18 @@ fn ingredients(w: &mut impl io::Write, recipe: &Recipe, converter: &Converter) -
 
         row.add_ansi_cell(format!("{content}{}", outcome_char.paint(outcome_style)));
 
-        if let Some(note) = &igr.note {
-            row.add_cell(format!("({note})"));
+        // Combine notes from all ingredients
+        let notes: Vec<_> = ingredients
+            .iter()
+            .filter_map(|igr| igr.note.as_ref())
+            .collect();
+        if !notes.is_empty() {
+            let combined_notes = notes
+                .iter()
+                .map(|n| n.as_str())
+                .collect::<Vec<_>>()
+                .join(", ");
+            row.add_cell(format!("({combined_notes})"));
         } else {
             row.add_cell("");
         }
@@ -378,8 +416,17 @@ fn steps(w: &mut impl io::Write, recipe: &Recipe) -> Result {
             match content {
                 cooklang::Content::Step(step) => {
                     let (step_text, step_ingredients) = step_text(recipe, section, step);
-                    let step_text = format!("{:>2}. {}", step.number, step_text.trim());
-                    print_wrapped_with_options(w, &step_text, |o| o.subsequent_indent("    "))?;
+                    let paragraphs: Vec<&str> = step_text.trim().split('\n').collect();
+                    for (i, paragraph) in paragraphs.iter().enumerate() {
+                        if i == 0 {
+                            let first = format!("{:>2}. {}", step.number, paragraph.trim_start());
+                            print_wrapped_with_options(w, &first, |o| o.subsequent_indent("    "))?;
+                        } else {
+                            print_wrapped_with_options(w, paragraph.trim_start(), |o| {
+                                o.initial_indent("    ").subsequent_indent("    ")
+                            })?;
+                        }
+                    }
                     print_wrapped_with_options(w, &step_ingredients, |o| {
                         let indent = "     "; // 5
                         o.initial_indent(indent)
@@ -402,10 +449,20 @@ fn steps(w: &mut impl io::Write, recipe: &Recipe) -> Result {
                         // Format as a note with a visual indicator
                         let note_style = yansi::Style::new().italic().fg(yansi::Color::Blue);
                         let note_prefix = "📝 Note: ".paint(note_style);
-                        write!(w, "  {note_prefix}")?;
-                        print_wrapped_with_options(w, t.trim(), |o| {
-                            o.initial_indent("").subsequent_indent("           ")
-                        })?;
+                        let note_indent = "           ";
+                        let paragraphs: Vec<&str> = t.trim().split('\n').collect();
+                        for (i, paragraph) in paragraphs.iter().enumerate() {
+                            if i == 0 {
+                                write!(w, "  {note_prefix}")?;
+                                print_wrapped_with_options(w, paragraph.trim(), |o| {
+                                    o.initial_indent("").subsequent_indent(note_indent)
+                                })?;
+                            } else {
+                                print_wrapped_with_options(w, paragraph.trim(), |o| {
+                                    o.initial_indent(note_indent).subsequent_indent(note_indent)
+                                })?;
+                            }
+                        }
                         writeln!(w)?;
                     }
                 }
