@@ -332,9 +332,9 @@ fn ingredients(
             }
         }
 
-        if ingredient.reference.is_some() {
+        if let Some(reference) = &ingredient.reference {
             let sep = std::path::MAIN_SEPARATOR.to_string();
-            let path = ingredient.reference.as_ref().unwrap().components.join(&sep);
+            let path = reference.components.join(&sep);
             write!(
                 w,
                 "[{}]({}{}{})",
@@ -379,7 +379,7 @@ fn cookware(
         write!(w, "- ").context("Failed to write cookware bullet")?;
         if !item.quantity.is_empty() {
             if opts.italic_amounts {
-                write!(w, "*{} * ", item.quantity).context("Failed to write italicized amount")?;
+                write!(w, "*{}* ", item.quantity).context("Failed to write italicized amount")?;
             } else {
                 write!(w, "{} ", item.quantity).context("Failed to write amount")?;
             }
